/**********************************************************************************************

Author : Mauro Laurenti
Version : 1.0
Date : 09/03/2012
 
CopyRight 2006-2012 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) 
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).  
The software is owned by the Author, and is protected under applicable copyright laws. 
All rights are reserved. 
Any use in violation of the foregoing restrictions may subject the 
user to criminal sanctions under applicable laws (Italian or International ones), as well as to 
civil liability for the breach of the terms and conditions of this license. 
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the following 
software doesn't aim to commercial use or monetary earning of any kind.    

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT, 
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

**********************************************************************************************/
									

#include "LCD_44780_I2C.h"
#include "PCF8574.h"
#include "delay.h"

unsigned char data_buffer = 0;
unsigned char PCF8574_address = 0;

//************************************************************
//                  Epulse  Implementation
//************************************************************
void Epulse (void) {
	int i;
	data_buffer = data_buffer | LCD_E;
	PCF8574_write_data (PCF8574_address, data_buffer);
	delay_ms (1);
	data_buffer = data_buffer & ~LCD_E;
	PCF8574_write_data (PCF8574_address, data_buffer);
}


//************************************************************
//                  SendCommand  Implementation
//************************************************************
void SendCommand (unsigned char data) {


	//clear the lower nible
	data_buffer = data_buffer & 0b11100000; 

	//write the data within the buffer
	data_buffer = data_buffer | data;

	PCF8574_write_data (PCF8574_address, data_buffer);
	Epulse ();
}


//************************************************************
//                  Line2LCD  Implementation
//************************************************************
void Line2LCD(void) {
	
	SendCommand (0b00001100);	
	SendCommand (0b00000000); 		
}


//************************************************************
//                  HomeLCD  Implementation
//************************************************************
void HomeLCD(void) {
	
	SendCommand (0b00000000);	
	SendCommand (0b00000010); 		
}


//************************************************************
//                  ShiftLCD  Implementation
//************************************************************
void ShiftLCD(char shift, char number_of_shift) {
	
	char i;
	
	for (i=0; i < number_of_shift; i++) {	
		SendCommand (0b00000001);	
		SendCommand (0b00001000 | shift); 	
	}	
}


//************************************************************
//                  ShiftCursorLCD  Implementation
//************************************************************
void ShiftCursorLCD(char shift, char number_of_shift){
	
	char i;
	
	for (i=0; i < number_of_shift; i++) {
		SendCommand (0b00000001);	
		SendCommand (0b00000000 | shift);
	} 		
}

//************************************************************
//                  GotoLineLCD  Implementation
//************************************************************
void GotoLineLCD (char line) {

switch(line) {

	case 1: SendCommand(0b00001000);
			SendCommand(0b00000000);
			break;

	case 2: SendCommand(0b00001100);
			SendCommand(0b00000000);
			break;

	case 3: SendCommand(0b00001001);
			SendCommand(0b00000100);
			break;

	case 4: SendCommand(0b00001101);
			SendCommand(0b00000100);
	}
}

//************************************************************
//                  WriteCharLCD  Implementation
//************************************************************
void WriteCharLCD (unsigned char value) {	
	
	unsigned char preliminary_buffer;
	
	data_buffer = data_buffer | LCD_RS;
	PCF8574_write_data (PCF8574_address, data_buffer);

	// Splitting of the first nibble
	preliminary_buffer = (value & 0xF0) >> 4;	
					
	SendCommand (preliminary_buffer);
		
	// Splitting of the second nibble
	preliminary_buffer = (value & 0x0F);
			
	SendCommand (preliminary_buffer);
		
	data_buffer = data_buffer & ~LCD_RS;
	PCF8574_write_data (PCF8574_address, data_buffer);
}

//************************************************************
//                  WriteStringLCD  Implementation
//************************************************************
void WriteStringLCD(const rom char *buffer) {
	
	 // Write data to LCD up to null
    while(*buffer) {
	    
	    // Write character to LCD
	    WriteCharLCD(*buffer);  
	    // Increment buffer
	    buffer++;              
   }
}

//************************************************************
//                  WriteVarLCD  Implementation
//************************************************************
void WriteVarLCD(unsigned char *buffer) {
	
	// Write data to LCD up to null
	while(*buffer){
		
		// Write character to LCD
		WriteCharLCD(*buffer); 
		// Increment buffer
		buffer++;               
	}
}


//************************************************************
//                  WriteIntLCD  Implementation
//************************************************************
void WriteIntLCD(int value, char number_of_digits){
	
	// The array size is 5 plus end of string \0
	unsigned char convertedInt [6];
	
	// Index used to shift to the right the digit
	char index; 
	
	// Integer is converted to string
	itoa (value, (char*) convertedInt);	
	
	if (number_of_digits >0 ) {
		
		convertedInt[number_of_digits] = '\0';
		
		// Shift the digit to the right removing the empty one
		while (!isdigit(convertedInt[number_of_digits-1])) {
			
			for (index = number_of_digits-1; index > 0; index--){
				convertedInt[index] = convertedInt[index-1];
				convertedInt[index-1] = ' ';
			}							
		}	
	}

	WriteVarLCD (convertedInt);
	
}	


//************************************************************
//                  ClearLCD  Implementation
//************************************************************
void ClearLCD (void){
	
	SendCommand (0b00000000);	
	SendCommand (0b00000001);		
}

//************************************************************
//                  CursorLCD  Implementation
//************************************************************
void CursorLCD(char active,char blinking) {	
	
	SendCommand (0b00000000);	
	SendCommand (0b00001100 | active | blinking);								
}

//************************************************************
//                  CursorLCD  Implementation
//************************************************************
void BacklightLCD(char active) {

	//Clear the LED bit
	data_buffer = data_buffer & ~TURN_ON_LED;

	//write the data within the buffer
	data_buffer = data_buffer | active;
	PCF8574_write_data (PCF8574_address, data_buffer);	
}

//************************************************************
//                  OpenLCD  Implementation
//************************************************************
void  OpenLCD(unsigned char quartz_frequency) {	
	
	setQuartz (quartz_frequency);

	PCF8574_address = PCF8574_ADDRESS_H | (PCF8574_ADDRESS_L << 1);

	data_buffer = 0;
	PCF8574_write_data (PCF8574_address, data_buffer);	
	
	delay_ms (100);
	SendCommand (0b00000011);
	delay_ms (5);	
	SendCommand (0b00000011);
	delay_ms (5);		
	SendCommand (0b00000011);
	delay_ms (5);		
	SendCommand (0b00000010);
	SendCommand (0b00000010);
	SendCommand (0b00001000);
	SendCommand (0b00000000);
	SendCommand (0b00001110);
	CursorLCD (TURN_OFF_CURSOR,BLINK_OFF);
	
	ClearLCD ();	
}

